@@ -36,8 +36,9 @@ import math
 import numpy
 
 # hwrt modules
-from . import HandwrittenData  # Necessary for pickle files
-from . import preprocessing  # Necessary for pickle files
+# HandwrittenData and preprocessing are needed because of pickle
+from . import HandwrittenData  # pylint: disable=W0611
+from . import preprocessing  # pylint: disable=W0611
 from . import utils
 
 
@@ -70,20 +71,7 @@ def get_class(name):
 
 def get_metrics(metrics_description):
     """Get metrics from a list of dictionaries. """
-    metric_list = []
-    for metric in metrics_description:
-        for feat, params in metric.items():
-            feat = get_class(feat)
-            if feat is not None:
-                if params is None:
-                    metric_list.append(feat())
-                else:
-                    parameters = {}
-                    for dicts in params:
-                        for param_name, param_value in dicts.items():
-                            parameters[param_name] = param_value
-                    metric_list.append(feat(**parameters))
-    return metric_list
+    return utils.get_objectlist(metrics_description, get_class)
 
 # Helper functions that are useful for some metrics
 
