@@ -25,7 +25,6 @@ import sys
 logging.basicConfig(format='%(asctime)s %(levelname)s %(message)s',
                     level=logging.DEBUG,
                     stream=sys.stdout)
-from shapely.geometry import LineString
 import itertools
 import numpy
 
@@ -33,6 +32,7 @@ import numpy
 from . import HandwrittenData
 from . import preprocessing
 from . import utils
+from . import geometry
 
 
 def get_class(name):
@@ -75,19 +75,7 @@ def get_features(model_description_features):
      - pen down feature: False
     ]
     """
-    feature_list = []
-    for feature in model_description_features:
-        for feat, params in feature.items():
-            feat = get_class(feat)
-            if params is None:
-                feature_list.append(feat())
-            else:
-                parameters = {}
-                for dicts in params:
-                    for param_name, param_value in dicts.items():
-                        parameters[param_name] = param_value
-                feature_list.append(feat(**parameters))
-    return feature_list
+    return utils.get_objectlist(model_description_features, get_class)
 
 # Only feature calculation classes follow
 # Everyone must have a __str__, __repr__, __call__ and get_dimension function
@@ -103,12 +91,12 @@ def get_features(model_description_features):
 
 class ConstantPointCoordinates(object):
 
-    """Take the first `points_per_stroke=20` points coordinates of the first
-       `strokes=4` strokes as features. This leads to
+    """Take the first ``points_per_stroke=20`` points coordinates of the first
+       ``strokes=4`` strokes as features. This leads to
        :math:`2 \cdot \text{points\_per\_stroke} \cdot \text{strokes}`
        features.
 
-       If `points` is set to 0, the first `points\_per\_stroke` point
+       If ``points`` is set to 0, the first ``points\_per\_stroke`` point
        coordinates and the \verb+pen_down+ feature is used. This leads to
        :math:`3 \cdot \text{points_per_stroke}` features."""
 
@@ -140,6 +128,8 @@ class ConstantPointCoordinates(object):
                 self.pen_down)
 
     def get_dimension(self):
+        """Get the dimension of the returned feature. This equals the number
+           of elements in the returned list of numbers."""
         if self.strokes > 0:
             return 2*self.strokes * self.points_per_stroke
         else:
@@ -219,6 +209,8 @@ class FirstNPoints(object):
                (self.n)
 
     def get_dimension(self):
+        """Get the dimension of the returned feature. This equals the number
+           of elements in the returned list of numbers."""
         return 2*self.n
 
     def __call__(self, handwritten_data):
@@ -257,6 +249,8 @@ class StrokeCount(object):
         return "stroke count"
 
     def get_dimension(self):
+        """Get the dimension of the returned feature. This equals the number
+           of elements in the returned list of numbers."""
         return 1
 
     def __call__(self, handwritten_data):
@@ -282,6 +276,8 @@ class Bitmap(object):
         return self.__repr__()
 
     def get_dimension(self):
+        """Get the dimension of the returned feature. This equals the number
+           of elements in the returned list of numbers."""
         return self.n**2
 
     def __call__(self, handwritten_data):
@@ -303,7 +299,7 @@ class Bitmap(object):
                                                           url=url,
                                                           folder=foldername)
         os.system(command)
-        from PIL import Image  # TODO: remove dependency for Python3
+        from PIL import Image
         im = Image.open("%s%i.png" % (foldername, raw_data_id))
         pix = im.load()
         # pixel_image = [[0 for i in range(28)] for j in range(28)]
@@ -332,6 +328,8 @@ class Ink(object):
         return "ink"
 
     def get_dimension(self):
+        """Get the dimension of the returned feature. This equals the number
+           of elements in the returned list of numbers."""
         return 1
 
     def __call__(self, handwritten_data):
@@ -345,7 +343,7 @@ class Ink(object):
             last_point = None
             for point in stroke:
                 if last_point is not None:
-                    ink += preprocessing._euclidean_distance(last_point, point)
+                    ink += preprocessing.euclidean_distance(last_point, point)
                 last_point = point
         return [ink]
 
@@ -363,6 +361,8 @@ class AspectRatio(object):
         return "Aspect Ratio"
 
     def get_dimension(self):
+        """Get the dimension of the returned feature. This equals the number
+           of elements in the returned list of numbers."""
         return 1
 
     def __call__(self, handwritten_data):
@@ -393,6 +393,8 @@ class Width(object):
         return "Width"
 
     def get_dimension(self):
+        """Get the dimension of the returned feature. This equals the number
+           of elements in the returned list of numbers."""
         return 1
 
     def __call__(self, handwritten_data):
@@ -421,6 +423,8 @@ class Height(object):
         return "Height"
 
     def get_dimension(self):
+        """Get the dimension of the returned feature. This equals the number
+           of elements in the returned list of numbers."""
         return 1
 
     def __call__(self, handwritten_data):
@@ -468,6 +472,8 @@ class CenterOfMass(object):
         return "Center of mass"
 
     def get_dimension(self):
+        """Get the dimension of the returned feature. This equals the number
+           of elements in the returned list of numbers."""
         return 2
 
     def __call__(self, handwritten_data):
@@ -499,6 +505,8 @@ class StrokeCenter(object):
         return "Stroke center"
 
     def get_dimension(self):
+        """Get the dimension of the returned feature. This equals the number
+           of elements in the returned list of numbers."""
         return self.strokes*2
 
     def __call__(self, handwritten_data):
@@ -556,6 +564,8 @@ class StrokeIntersections(object):
         return "StrokeIntersections"
 
     def get_dimension(self):
+        """Get the dimension of the returned feature. This equals the number
+           of elements in the returned list of numbers."""
         return int(round(float(self.strokes**2)/2 + float(self.strokes)/2))
 
     def __call__(self, handwritten_data):
@@ -563,69 +573,24 @@ class StrokeIntersections(object):
             "handwritten data is not of type HandwrittenData, but of %r" % \
             type(handwritten_data)
 
-        def count_stroke_selfintersections(stroke):
-            """ Get the number of self-intersections of a stroke."""
-            # This can be solved more efficiently with sweep line
-            counter = 0
-            lines = []
-            for last, point in zip(stroke, stroke[1:]):
-                line = LineString([(last['x'], last['y']),
-                                   (point['x'], point['y'])])
-                lines.append((len(lines), line))
+        pointlist = handwritten_data.get_pointlist()
+        polygonalChains = []
 
-            for line1, line2 in itertools.combinations(lines, 2):
-                index1, line1 = line1
-                index2, line2 = line2
-                if line1.intersects(line2) and abs(index1 - index2) > 1:
-                    counter += 1
-            return counter
-
-        def count_intersections(stroke_a, stroke_b):
-            counter = 0
-
-            # build data structure a
-            lines_a = []
-            for last, point in zip(stroke_a, stroke_a[1:]):
-                line = LineString([(last['x'], last['y']),
-                                   (point['x'], point['y'])])
-                lines_a.append(line)
-
-            # build data structure b
-            lines_b = []
-            for last, point in zip(stroke_b, stroke_b[1:]):
-                line = LineString([(last['x'], last['y']),
-                                   (point['x'], point['y'])])
-                lines_b.append(line)
-
-            # Calculate intersections
-            for line1, line2 in itertools.product(lines_a, lines_b):
-                if line1.intersects(line2):
-                    counter += 1
-            return counter
+        # Make sure the dimension is correct
+        for i in range(self.strokes):
+            if i >= self.strokes:
+                break
+            if i < len(pointlist):
+                polygonalChains.append(geometry.PolygonalChain(pointlist[i]))
+            else:
+                polygonalChains.append(None)
 
         x = []
-        pointlist = handwritten_data.get_pointlist()
-        for i in range(self.strokes):
-            for j in range(i, self.strokes):
-                if len(pointlist) <= i or len(pointlist) <= j:
-                    # There are less strokes! If there is no stroke, nothing
-                    # can intersect
-                    x.append(0)
-                else:
-                    # Does stroke i intersect stroke j?
-                    if i == j:
-                        x.append(count_stroke_selfintersections(pointlist[i]))
-                    else:
-                        x.append(count_intersections(pointlist[i],
-                                                     pointlist[j]))
-        # print("%s - %i" % (handwritten_data.formula_in_latex,
-        #                    handwritten_data.raw_data_id))
-        # curr = 0
-        # for i in range(self.strokes, 0, -1):
-        #     line = "\t"*(self.strokes - i)
-        #     for j in range(i):
-        #         line += str(x[curr]) + "\t"
-        #         curr += 1
+        for chainA, chainB in itertools.combinations_with_replacement(polygonalChains, 2):
+            if chainA == chainB:
+                x.append(chainA.count_selfintersections())
+            else:
+                x.append(chainA.count_intersections(chainB))
 
         assert self.get_dimension() == len(x), \
             "Dimension of %s should be %i, but was %i" % \
@@ -656,6 +621,8 @@ class ReCurvature(object):
         return "Re-curvature"
 
     def get_dimension(self):
+        """Get the dimension of the returned feature. This equals the number
+           of elements in the returned list of numbers."""
         return self.strokes
 
     def __call__(self, handwritten_data):
@@ -668,7 +635,7 @@ class ReCurvature(object):
             height = max(stroke_y) - min(stroke_y)
             length = 0.0
             for last_point, point in zip(stroke, stroke[1:]):
-                length += preprocessing._euclidean_distance(point, last_point)
+                length += preprocessing.euclidean_distance(point, last_point)
 
             if length == 0:
                 x.append(1)
