@@ -519,3 +519,25 @@ def classify_single_recording(raw_data_json, model_folder, verbose=False):
         results.append((index2latex[index], probability))
     results = sorted(results, key=lambda n: n[1], reverse=True)
     return results
+
+
+def get_objectlist(description, get_class):
+    """Take a description and return a list of classes.
+    :param description: list of dictionaries, where each dictionary has only
+                        one entry. The key is the name of a class.
+                        The value of that entry is a list of
+                        dictionaries again. Those dictionaries are paramters.
+    :returns: List of objects."""
+    object_list = []
+    for feature in description:
+        for feat, params in feature.items():
+            feat = get_class(feat)
+            if params is None:
+                object_list.append(feat())
+            else:
+                parameters = {}
+                for dicts in params:
+                    for param_name, param_value in dicts.items():
+                        parameters[param_name] = param_value
+                object_list.append(feat(**parameters))  # pylint: disable=W0142
+    return object_list
